@@ -415,6 +415,9 @@ func (b *LocalBackend) getServeHandler(r *http.Request) (_ ipn.HTTPHandlerView,
 	hostname := r.Host
 	if r.TLS == nil {
 		tcd := "." + b.Status().CurrentTailnet.MagicDNSSuffix
+		if host, _, err := net.SplitHostPort(hostname); err == nil {
+			hostname = host
+		}
 		if !strings.HasSuffix(hostname, tcd) {
 			hostname += tcd
 		}
